/*
    Парус 8 - Панели мониторинга - УДП - Доски задач
    Компонент: Диалоговое окно фильтра отбора
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useState, useContext, useEffect, useCallback } from "react"; //Классы React
import { BackEndСtx } from "../../../context/backend"; //Контекст взаимодействия с сервером
import PropTypes from "prop-types"; //Контроль свойств компонента
import {
    Dialog,
    DialogTitle,
    IconButton,
    Icon,
    DialogContent,
    DialogActions,
    Button,
    Box,
    Stack,
    Checkbox,
    FormControlLabel,
    Radio,
    RadioGroup
} from "@mui/material"; //Интерфейсные компоненты
import { FilterInputField } from "./filter_input_field"; //Компонент поля ввода
import { ApplicationСtx } from "../../../context/application"; //Контекст приложения
import { hasValue } from "../../../core/utils"; //Вспомогательные функции
import { APP_STYLES } from "../../../../app.styles"; //Типовые стили

//---------
//Константы
//---------

//Перечисление "Состояние события"
export const EVENT_STATES = Object.freeze({ 0: "Все", 1: "Не аннулированные", 2: "Аннулированные" });

//Стили
const STYLES = {
    FILTERS_SCROLL: { overflowY: "auto", ...APP_STYLES.SCROLL },
    DIALOG_ACTIONS: { justifyContent: "end", paddingRight: "24px", paddingLeft: "24px" },
    CLOSE_BUTTON: {
        position: "absolute",
        right: 8,
        top: 8,
        color: theme => theme.palette.grey[500]
    },
    DOCLINK_STACK: { alignItems: "baseline" },
    SELECT: { width: "450px" },
    BOX_WITH_LEGEND: { border: "1px solid #939393" },
    LEGEND: { textAlign: "left" }
};

//-----------------------
//Вспомогательные функции
//-----------------------

//Выбор типа события
const selectEventType = (value, showDictionary, callBack) => {
    showDictionary({
        unitCode: "ClientEventTypes",
        showMethod: "dictionary",
        inputParameters: [{ name: "pos_eventtypecode", value: value }],
        callBack: res => (res.success === true ? callBack(res.outParameters.eventtypecode) : callBack(null))
    });
};

//Выбор каталога
const selectCatalog = (value, showDictionary, callBack) => {
    showDictionary({
        unitCode: "CatalogTree",
        inputParameters: [
            { name: "in_DOCNAME", value: "ClientEvents" },
            { name: "in_NAME", value: value }
        ],
        callBack: res => (res.success === true ? callBack(res.outParameters.out_NAME) : callBack(null))
    });
};

//Выбор исполнителя
const selectSendPerson = (value, showDictionary, callBack) => {
    showDictionary({
        unitCode: "AGNLIST",
        showMethod: "agents",
        inputParameters: [{ name: "pos_agnmnemo", value: value }],
        callBack: res => (res.success === true ? callBack(res.outParameters.agnmnemo) : callBack(null))
    });
};

//Выбор подразделения
const selectSendDivision = (value, showDictionary, callBack) => {
    showDictionary({
        unitCode: "INS_DEPARTMENT",
        inputParameters: [{ name: "in_CODE", value: value }],
        callBack: res => (res.success === true ? callBack(res.outParameters.out_CODE) : callBack(null))
    });
};

//Выбор группы пользователей
const selectSendUsrGrp = (value, showDictionary, callBack) => {
    showDictionary({
        unitCode: "CostStaffGroups",
        inputParameters: [{ name: "in_CODE", value: value }],
        callBack: res => (res.success === true ? callBack(res.outParameters.out_CODE) : callBack(null))
    });
};

//---------------
//Тело компонента
//---------------

//Диалоговое окно фильтра отбора
const FilterDialog = ({ initial, docs, onCancel, onOk, getDocLinks }) => {
    //Собственное состояние
    const [filter, setFilter] = useState({ ...initial });

    //Состояние текущего типа события
    const [curType, setCurType] = useState(initial.type);

    //Состояние учётных документов
    const [curDocLinks, setCurDocLinks] = useState(docs);

    //Подключение к контексту взаимодействия с сервером
    const { executeStored } = useContext(BackEndСtx);

    //Получение подкаталогов
    const getSubCatalogs = useCallback(async () => {
        const data = await executeStored({
            stored: "PKG_P8PANELS_CLNTTSKBRD.CLNEVENTS_SUBCATALOGS_GET",
            args: {
                SNAME: filter.catalog,
                NSUBCAT: filter.wSubcatalogs ? 1 : 0
            }
        });
        return data.SRESULT;
    }, [executeStored, filter.catalog, filter.wSubcatalogs]);

    //Подключение к контексту приложения
    const { pOnlineShowDictionary } = useContext(ApplicationСtx);

    //При закрытии диалога без изменения фильтра
    const handleCancel = () => (onCancel ? onCancel() : null);

    //При очистке фильтра
    const handleClear = () => {
        setFilter({
            evState: EVENT_STATES[1],
            type: "",
            catalog: "",
            crn: "",
            wSubcatalogs: false,
            sendPerson: "",
            sendDivision: "",
            sendUsrGrp: "",
            docLink: ""
        });
    };

    //При закрытии диалога с изменением фильтра
    const handleOK = async () => {
        if (onOk) {
            if (filter.catalog && !filter.crn) {
                const crns = await getSubCatalogs();
                let filterCopy = { ...filter };
                crns ? (filterCopy.crn = crns) : null;
                onOk(filterCopy);
            } else onOk(filter);
        }
    };

    //При изменении значения элемента
    const handleFilterItemChange = (item, value) => {
        item === "type" && filter.docLink ? setFilter(pv => ({ ...pv, [item]: value, docLink: "" })) : setFilter(pv => ({ ...pv, [item]: value }));
    };

    //Очистка учётного документа
    const clearDocLink = () => setFilter(pv => ({ ...pv, docLink: "" }));

    //При изменении типа события
    useEffect(() => {
        if (curType) {
            if (curType !== filter.type) {
                clearDocLink();
                setCurDocLinks([]);
            } else if (curType === filter.type && curType === initial.type && !curDocLinks.length) setCurDocLinks(docs);
        }
    }, [curDocLinks, curType, docs, filter.type, initial.type]);

    //Обработка изменений с каталогами
    useEffect(() => {
        if (!filter.catalog && filter.wSubcatalogs) setFilter(pv => ({ ...pv, wSubcatalogs: false }));
        if (filter.catalog !== initial.catalog && filter.crn) setFilter(pv => ({ ...pv, crn: "" }));
        if (filter.catalog === initial.catalog && filter.crn !== initial.crn && filter.wSubcatalogs === initial.wSubcatalogs)
            setFilter(pv => ({ ...pv, crn: initial.crn }));
        if (filter.catalog === initial.catalog && filter.wSubcatalogs !== initial.wSubcatalogs && !filter.wSubcatalogs)
            setFilter(pv => ({ ...pv, crn: initial.crn.split(";")[0] }));
        if (filter.catalog === initial.catalog && filter.wSubcatalogs !== initial.wSubcatalogs && filter.wSubcatalogs)
            setFilter(pv => ({ ...pv, crn: "" }));
    }, [filter.catalog, filter.crn, filter.wSubcatalogs, initial.catalog, initial.crn, initial.wSubcatalogs]);

    //Генерация содержимого
    return (
        <div>
            <Dialog open onClose={handleCancel} fullWidth maxWidth="sm">
                <DialogTitle>Фильтр отбора</DialogTitle>
                <IconButton aria-label="close" onClick={handleCancel} sx={STYLES.CLOSE_BUTTON}>
                    <Icon>close</Icon>
                </IconButton>
                <DialogContent sx={STYLES.FILTERS_SCROLL}>
                    <Box sx={STYLES.BOX_WITH_LEGEND} component="fieldset">
                        <legend style={STYLES.LEGEND}>Состояние</legend>
                        <RadioGroup
                            row
                            aria-labelledby="evState-label"
                            id="evState"
                            name="evState"
                            value={filter.evState}
                            onChange={e => handleFilterItemChange(e.target.name, e.target.value)}
                        >
                            {Object.keys(EVENT_STATES).map(function (k) {
                                return <FormControlLabel key={k} value={EVENT_STATES[k]} control={<Radio />} label={EVENT_STATES[k]} />;
                            })}
                        </RadioGroup>
                    </Box>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="type"
                            elementValue={filter.type}
                            labelText="Тип"
                            dictionary={callBack => selectEventType(filter.type, pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                    </Box>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="catalog"
                            elementValue={filter.catalog}
                            labelText="Каталог"
                            dictionary={callBack => selectCatalog(filter.catalog, pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                        <FormControlLabel
                            control={
                                <Checkbox
                                    id="wSubcatalogs"
                                    name="wSubcatalogs"
                                    checked={filter.wSubcatalogs}
                                    disabled={filter.catalog ? false : true}
                                    onChange={e => handleFilterItemChange(e.target.name, e.target.checked)}
                                />
                            }
                            label="Включая подкаталоги"
                        />
                    </Box>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="sendPerson"
                            elementValue={filter.sendPerson}
                            labelText="Исполнитель"
                            dictionary={callBack => selectSendPerson(filter.sendPerson, pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                    </Box>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="sendDivision"
                            elementValue={filter.sendDivision}
                            labelText="Подразделение"
                            dictionary={callBack => selectSendDivision(filter.sendDivision, pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                    </Box>
                    <Box component="section" p={1}>
                        <FilterInputField
                            elementCode="sendUsrGrp"
                            elementValue={filter.sendUsrGrp}
                            labelText="Группа пользователей"
                            dictionary={callBack => selectSendUsrGrp(filter.sendUsrGrp, pOnlineShowDictionary, callBack)}
                            onChange={handleFilterItemChange}
                        />
                    </Box>
                    <Box component="section" p={1}>
                        <Stack direction="row" sx={STYLES.DOCLINK_STACK}>
                            <FilterInputField
                                elementCode="docLink"
                                elementValue={filter.docLink}
                                labelText="Учётный документ"
                                items={curDocLinks}
                                disabled={!curDocLinks.length ? true : false}
                                onChange={handleFilterItemChange}
                                sx={STYLES.SELECT}
                            />
                            <IconButton title="Очистить" disabled={!filter.docLink} onClick={clearDocLink}>
                                <Icon>clear</Icon>
                            </IconButton>
                            <IconButton
                                title="Обновить"
                                disabled={!((!curType || curType !== filter.type) && filter.type)}
                                onClick={() => {
                                    setCurType(filter.type);
                                    clearDocLink();
                                    getDocLinks(filter.type).then(dl => {
                                        setCurDocLinks(dl);
                                        console.log(dl);
                                    });
                                }}
                            >
                                <Icon>refresh</Icon>
                            </IconButton>
                        </Stack>
                    </Box>
                </DialogContent>
                <DialogActions sx={STYLES.DIALOG_ACTIONS}>
                    <Button disabled={!hasValue(filter.type)} variant="text" onClick={handleOK}>
                        ОК
                    </Button>
                    <Button variant="text" onClick={handleClear}>
                        Очистить
                    </Button>
                    <Button variant="text" onClick={handleCancel}>
                        Отмена
                    </Button>
                </DialogActions>
            </Dialog>
        </div>
    );
};

//Контроль свойств компонента - Диалоговое окно фильтра отбора
FilterDialog.propTypes = {
    initial: PropTypes.object.isRequired,
    docs: PropTypes.arrayOf(PropTypes.object),
    onOk: PropTypes.func,
    onCancel: PropTypes.func,
    getDocLinks: PropTypes.func
};

//--------------------
//Интерфейс компонента
//--------------------

export { FilterDialog };
